@@ -1,3 +1,7 @@
+#  Authors: Sylvain MARIE <sylvain.marie@se.com>
+#            + All contributors to <https://github.com/smarie/python-odsclient>
+#
+#  License: 3-clause BSD, <https://github.com/smarie/python-odsclient/blob/master/LICENSE>
 import warnings
 from ast import literal_eval
 from getpass import getpass
@@ -8,10 +12,10 @@ from threading import Lock
 
 try:
     # Python 3
-    from urllib.parse import urlparse, parse_qs
+    from urllib.parse import urlparse
 except ImportError:
     # Python 2
-    from urlparse import urlparse, parse_qs
+    from urlparse import urlparse
 
 try:
     from pathlib import Path
@@ -44,15 +48,6 @@ except NameError:
 
 from requests import Session, HTTPError
 
-try:
-    # python 3
-    # noinspection PyCompatibility
-    from urllib.parse import quote
-except ImportError:
-    # python 2
-    # noinspection PyUnresolvedReferences
-    from urllib import quote
-
 try:
     # noinspection PyUnresolvedReferences
     from typing import Dict, Union, Iterable
@@ -474,15 +469,21 @@ class ODSClient(object):
             try:
                 import pandas as pd
             except ImportError as e:
-                raise Exception("`push_dataset_realtime` with the `pandas` format requires `pandas` to be installed. [%s] %s" % (e.__class__, e))
-            # noinspection PyStatementEffect
+                raise Exception("`push_dataset_realtime` with the `pandas` format requires `pandas` to be installed. "
+                                "[%s] %s" % (e.__class__, e))
+
+            if not isinstance(dataset, pd.DataFrame):
+                raise TypeError('If format is set to "pandas" then `dataset` should be a DataFrame. Found %s'
+                                % type(dataset))
+
             dataset  # type:pandas.DataFrame
             request_body = dataset.to_json(orient='records')
         elif format == 'csv':
             try:
                 import csv
             except ImportError as e:
-                raise Exception("`push_dataset_realtime` with the `csv` format requires `csv` to be installed. [%s] %s" % (e.__class__, e))
+                raise Exception("`push_dataset_realtime` with the `csv` format requires `csv` to be installed. [%s] %s"
+                                % (e.__class__, e))
             # noinspection PyStatementEffect
             dataset  # type:str
             csv_reader = csv.DictReader(StringIO(dataset), delimiter=csv_separator)
@@ -530,13 +531,28 @@ class ODSClient(object):
         import keyring
         keyring.delete_password(self.base_url, self.keyring_entries_username)
 
-    def get_apikey_from_keyring(self):
+    def get_apikey_from_keyring(self, ignore_import_errors=False):
         """
         Looks for a keyring entry containing the api key and returns it.
         If not found, returns `None`
+
+        :param ignore_import_errors: when this is `True`, the method will return `None` if `keyring` is not installed
+            instead of raising an `ImportError`.
         :return:
         """
-        import keyring
+        if ignore_import_errors:
+            try:
+                import keyring
+            except ImportError as e:
+                # not installed: simply warn instead of raising exception
+                warnings.warn("`keyring` is not installed but the `ODSClient` is configured to use it. You can either"
+                              "set `use_keyring=False` explicitly or install keyring to disable this warning. "
+                              "Caught: %r" % e)
+                return None
+        else:
+            # do not catch any exception
+            import keyring
+
         for _url in (self.base_url, self.base_url + '/'):
             apikey = keyring.get_password(_url, self.keyring_entries_username)
             if apikey is not None:
@@ -601,9 +617,9 @@ class ODSClient(object):
         if self.apikey is not None:
             return self.apikey
 
-        # 2- if keyring service contains an entry, use it
+        # 2- if keyring service is installed and contains an entry, use it
         if self.use_keyring:
-            apikey = self.get_apikey_from_keyring()
+            apikey = self.get_apikey_from_keyring(ignore_import_errors=True)
             if apikey is not None:
                 return apikey
 
@@ -924,7 +940,8 @@ class CacheEntry(object):
     def assert_exists(self):
         """Raises an error if the file does not exist"""
         if not self.exists():
-            raise CacheFileNotFoundError("Cached file entry can not be read as it does not exist: '%s'" % self.file_path)
+            raise CacheFileNotFoundError("Cached file entry can not be read as it does not exist: '%s'"
+                                         % self.file_path)
 
     def read(self):
         # type: (...) -> str
